
{*******************************************************}
{                                                       }
{  Name:  GPIB.PAS                                   }
{                                                       }
{   GPIB Interface Unit for Borland Delphi           }
{                                                       }
{ (c) Copyright 1998                                    }
{ All Rights Reserved                                   }
{                                                       }
{*******************************************************}

unit GPIB;

{$D+,S+,L+}

interface

{ GPIB Commands codes }
const
   GTL = $01;
   SDC = $04;
   PPC = $05;
   GET = $08;
   TCT = $09;
   LLO = $11;
   DCL = $14;
   PPU = $15;
   SPE = $18;
   SPD = $19;
   UNL = $3f;
   UNT = $5f;
   PPE = $60;
   PPD = $70;


{ Bit specifiers for IBSTA status variable and wait mask  }
   ERR  = $8000;   { Error                           }
   TIMO = $4000;   { Timeout                         }
   EEND = $2000;   { EOI or EOS detected             }
   SRQI = $1000;   { SRQ detected by CIC             }
   RQS  = $0800;   { Device needs service            }
   SPOLL= $0400;   { Board has been serially polled  }
   EVENT= $0200;   { An event has occured            }
   CMPL = $0100;   { I/O completed                   }
   LOK  = $0080;   { Local lockout state             }
   REM  = $0040;   { Remote state                    }
   CIC  = $0020;   { Controller-in-Charge            }
   ATN  = $0010;   { Attention asserted              }
   TACS = $0008;   { Talker active                   }
   LACS = $0004;   { Listener active                 }
   DTAS = $0002;   { Device trigger state            }
   DCAS = $0001;   { Device clear state              }



{ System error codes - returned in IBERR variable }
   EDVR  = 0;     { DOS error                                }
   ECIC  = 1;     { Board must be CIC                        }
   ENOL  = 2;     { No listeners detected                    }
   EADR  = 3;     { Board not addressed correctly            }
   EARG  = 4;     { Invalid argument specified               }
   ESAC  = 5;     { Board should be system controller        }
   EABO  = 6;     { I/O operation aborted                    }
   ENEB  = 7;     { Invalid interface board specified        }
   EOIP  = 10;    { I/O operation already running            }
   ECAP  = 11;    { Board does not have requested capability }
   EFSO  = 12;    { Error returned from file system          }
   EBUS  = 14;    { Command error on bus                     }
   ESTB  = 15;    { Serial poll responde byte lost           }
   ESRQ  = 16;    { SRQ is still asserted                    }
   ETAB  = 20;    { No device responding with ETAB           }
   EINT  = 247;   { No interrupt configured on baord         }
   ECFG  = 252;   { Board type doesn't match GPIB.CFG        }


{ EOS mode bits   }
   BIN  = $1000;
   XEOS = $0800;
   REOS = $0400;

{ Timeout values and meanings }
   TNONE  =  0;   { No timeout         }
   T10us  =  1;   { 10   microseconds  }
   T30us  =  2;   { 30   microseconds  }
   T100us =  3;   { 100  microseconds  }
   T300us =  4;   { 300  microseconds  }
   T1ms   =  5;   { 1    milliseconds  }
   T3ms   =  6;   { 3    milliseconds  }
   T10ms  =  7;   { 10   milliseconds  }
   T30ms  =  8;   { 30   milliseconds  }
   T100ms =  9;   { 100  milliseconds  }
   T300ms = 10;   { 300  milliseconds  }
   T1s    = 11;   { 1    seconds  }
   T3s    = 12;   { 3    seconds  }
   T10s   = 13;   { 10   seconds  }
   T30s   = 14;   { 30   seconds  }
   T100s  = 15;   { 100  seconds  }
   T300s  = 16;   { 300  seconds  }
   T1000s = 17;   { 1000 seconds  }


{ IBLN contants }
   NO_SAD  = 0;     { No secondary address    }
   ALL_SAD = -1;    { All secondary addresses }


{ Miscellaneous  }
   S   = $08;           { parallel poll bit  }
   LF  = $0a;           { linefeed character }
   NOADDR    = $FFFF;   { Terminator for address list    }
   NULLend   = 0;       { Send() EOTMODE - Do nothing at the end of a transfe   }
   NLend     = 1;       { Send() EOTMODE - Send NL with EOI after a transfer.   }
   DABend    = 02;      { Send() EOTMODE - Send EOI with the last DAB. }
   STOPend   = $0100;   { Receive() termination }
   EventDTAS = 1;       { used by IBEVENT()   }
   EventDCAS = 2;       { used by IBEVENT()   }


{  These constants are used with ibconfig() to specify one of the
    configurable options. }
   IbcPAD =           $0001;    { Primary Address                  }
   IbcSAD =           $0002;    { Secondary Address                }
   IbcTMO =           $0003;    { Timeout                          }
   IbcEOT =           $0004;    { Send EOI with last byte          }
   IbcPPC =           $0005;    { Parallel Poll configure          }
   IbcREADDR =        $0006;    { Repeat addressing                }
   IbcAUTOPOLL =      $0007;    { Disable automatic serial polling }
   IbcCICPROT =       $0008;    { Use CIC Protocol                 }
   IbcIRQ =           $0009;    { Interrupt level (or 0 for NONE)  }
   IbcSC =            $000A;    { Board is ssystem controller      }
   IbcSRE =           $000B;    { Assert SRE for device calls      }
   IbcEOSrd =         $000C;    { Terminate read on EOS            }
   IbcEOSwrt =        $000D;    { Send EOI with EOS                }
   IbcEOScmp =        $000E;    { Use 7 or 8-bit compare on EOS    }
   IbcEOSchar =       $000F;    { EOS character                    }
   IbcPP2 =           $0010;    { Use PP Mode 2                    }
   IbcTIMING =        $0011;    { Normal, High or VeryHighspeed    }
   IbcDMA =           $0012;    { DMA channle (or 0 for NONE)      }
   IbcReadAdjust =    $0013;    { Swap bytes on read               }
   IbcWriteAdjust =   $0014;    { Swap bytes on write              }
   IbcEventQueue =    $0015;    { Use event queue                  }
   IbcSPollBit =      $0016;    { Serial poll bit used             }
   IbcSendLLO =       $0017;    { Automatically send LLO           }
   IbcSPollTime =     $0018;    { Serial polls timeout             }
   IbcPPollTime =     $0019;    { Parallel poll timeout            }
   IbcEndBitIsNormal = $001A;   { Don't set END bit on EOS.        }
   IbcUnAddr     =    $001B;    { UN-ADDRESSING OPTION             }
   IbcIst        =    $0020;    { CURRENT PARALLEL POLL STATUS BIT }
   IbcRsv        =    $0021;    { CURRENT SERIAL POLL BYTE         }
   IbcBNA =           $0200;    { A device's access board.         }
   IbcBaseAddr =      $0201;    { A GPIB board's base I/O address. }
   IbcDMAChn    =     $0202;    { ASSIGNED DMA CHANNEL             }
   IbcIRQLev    =     $0203;    { ASSIGNED IRQ LEVEL               }

   { CBI Config Items - don't exist in NI software }
   IbcBoardType =     $0300;    { Board type                       }
   IbcChipType =      $0301;    { Chip type                        }
   IbcSlotNum =       $0302;    { Slot Number                      }
   IbcPCIIndex =      $0303;    { PCI Index                        }
   IbcBaseAdr2 =      $0304;    { 2nd Base address for PCI board   }
   IbcUseFIFOs =      $0305;    { When to use FIFOs   }


{  These constants are used with ibask() to specify one of the
    configurable options. }
   IbaPAD =           $0001;    { Primary Address                  }
   IbaSAD =           $0002;    { Secondary Address                }
   IbaTMO =           $0003;    { Timeout                          }
   IbaEOT =           $0004;    { Send EOI with last byte          }
   IbaPPC =           $0005;    { Parallel Poll configure          }
   IbaREADDR =        $0006;    { Repeat addressing                }
   IbaAUTOPOLL =      $0007;    { Disable automatic serial polling }
   IbaCICPROT =       $0008;    { Use CIC Protocol                 }
   IbaIRQ =           $0009;    { Interrupt level (or 0 for NONE)  }
   IbaSC =            $000A;    { Board is ssystem controller      }
   IbaSRE =           $000B;    { Assert SRE for device calls      }
   IbaEOSrd =         $000C;    { Terminate read on EOS            }
   IbaEOSwrt =        $000D;    { Send EOI with EOS                }
   IbaEOScmp =        $000E;    { Use 7 or 8-bit compare on EOS    }
   IbaEOSchar =       $000F;    { EOS character                    }
   IbaPP2 =           $0010;    { Use PP Mode 2                    }
   IbaTIMING =        $0011;    { Normal, High or VeryHighspeed    }
   IbaDMA =           $0012;    { DMA channle (or 0 for NONE)      }
   IbaReadAdjust =    $0013;    { Swap bytes on read               }
   IbaWriteAdjust =   $0014;    { Swap bytes on write              }
   IbaEventQueue =    $0015;    { Use event queue                  }
   IbaSPollBit =      $0016;    { Serial poll bit used             }
   IbaSendLLO =       $0017;    { Automatically send LLO           }
   IbaSPollTime =     $0018;    { Serial polls timeout             }
   IbaPPollTime =     $0019;    { Parallel poll timeout            }
   IbaEndBitIsNormal = $001A;   { Don't set END bit on EOS.        }
   IbaUnAddr     =    $001B;    { UN-ADDRESSING OPTION             }
   IbaIst        =    $0020;    { CURRENT PARALLEL POLL STATUS BIT }
   IbaRsv        =    $0021;    { CURRENT SERIAL POLL BYTE         }
   IbaBNA =           $0200;    { A device's access board.         }
   IbaBaseAddr =      $0201;    { A GPIB board's base I/O address. }
   IbcDMAChannel  =   $0202;    { ASSIGNED DMA CHANNEL             }
   IbcIRQLevel  =     $0203;    { ASSIGNED IRQ LEVEL               }

   { CBI Config Items - don't exist in NI software }
   IbaBoardType =     $0300;    { Board type                       }
   IbaChipType =      $0301;    { Chip type                        }
   IbaSlotNum =       $0302;    { Slot Number                      }
   IbaPCIIndex =      $0303;    { PCI Index                        }
   IbaBaseAdr2 =      $0304;    { 2nd Base address for PCI board   }
   IbaUseFIFOs =      $0305;    { When to use FIFOs                }


{ These bits specify which lines can be monitored by IBLINES }
  ValidDAV = $0001;
  ValidNDAC = $0002;
  ValidNRFD = $0004;
  ValidIFC = $0008;
  ValidREN = $0010;
  ValidSRQ = $0020;
  ValidATN = $0040;
  ValidEOI = $0080;

{ These bits specify the current state of each GPIB line }
  BusDAV = $0100;
  BusNDAC = $0200;
  BusNRFD = $0400;
  BusIFC = $0800;
  BusREN = $1000;
  BusSRQ = $2000;
  BusATN = $4000;
  BusEOI = $8000;




{***************************************************************************}

var
{ GPIB global  variables }
  ibsta: word;
  iberr: word;
  ibcnt: word;
  ibcntl: Longint;



{***************************************************************************}

{$IFDEF WIN32}
{        32-bit function prototypes }

procedure ibask    (board:Integer; item:Integer; var value:Integer); stdcall;
procedure ibbna    (device:Integer; boardname:PChar); stdcall;
procedure ibcac    (device:Integer; sync:Integer); stdcall;
procedure ibclr    (device:Integer); stdcall;
procedure ibcmd    (board:Integer; cmnd:PChar; bytecount:Longint); stdcall;
procedure ibcmda   (board:Integer; cmnd:Pchar; bytecount:Longint); stdcall;
procedure ibconfig (board:Integer; item:Integer; value:Integer); stdcall;
function  ibdev    (boardindex:Integer; pad:Integer; sad:Integer;
                    timeout:Integer; eot:Integer; eos:Integer):Integer; stdcall;
procedure ibdma    (device:Integer; dma:Integer); stdcall;
procedure ibeos    (boarddev:Integer; eos:Integer); stdcall;
procedure ibeot    (boarddev:Integer; eot:Integer); stdcall;
procedure ibevent  (board:Integer; var event:Integer); stdcall;
function  ibfind   (name:PChar):Integer; stdcall;
procedure ibgts    (board:Integer; handshake:Integer); stdcall;
procedure ibinit   (board:Integer); stdcall;
procedure ibist    (board:Integer; Statusbit:Integer); stdcall;
procedure iblines  (board:Integer; var clines:Integer); stdcall;
procedure ibln     (board:Integer; pad:Integer; sad:Integer; var listenflag:Integer); stdcall;
procedure ibloc    (boarddev:Integer); stdcall;
procedure ibonl    (boarddev:Integer; online:Integer); stdcall;
procedure ibpad    (boarddev:Integer; address:Integer); stdcall;
procedure ibpct    (device:Integer); stdcall;
procedure ibppc    (board:Integer; command:Integer); stdcall;
procedure ibptrs   (var ibstaptr:word; var iberrPtr:word; var ibcntPtr:word; var ibcntlPtr:Longint); stdcall;
procedure ibrd     (boarddev:Integer; readstr:Pchar; bytecount:Longint); stdcall;
procedure ibrda    (boarddev:Integer; readstr:Pchar; bytecount:Longint); stdcall;
procedure ibrdf    (boarddev:Integer; filename:PChar); stdcall;
procedure ibrdi    (boarddev:Integer; var readval:word; bytecount:Longint); stdcall;
procedure ibrdia   (boarddev:Integer; var readval:word; bytecount:Longint); stdcall;
procedure ibrpp    (boarddev:Integer; var response:Byte); stdcall;
procedure ibrsc    (board:Integer; control:Integer); stdcall;
procedure ibrsp    (device:Integer; var serialpollbyte:Byte); stdcall;
procedure ibrsv    (boarddev:Integer; statusbyte:Integer); stdcall;
procedure ibsad    (boarddev:Integer; address:Integer); stdcall;
procedure ibsic    (board:Integer); stdcall;
procedure ibsre    (board:Integer; ren:Integer); stdcall;
procedure ibstop   (boarddev:Integer); stdcall;
procedure ibtmo    (boarddev:Integer; timeout:Integer); stdcall;
procedure ibtrg    (device:Integer); stdcall;
procedure ibwait   (boarddev:Integer; mask:word); stdcall;
procedure ibwrt    (boarddev:Integer; writestr:Pchar; bytecount:Longint); stdcall;
procedure ibwrta   (boarddev:Integer; writestr:Pchar; bytecount:Longint); stdcall;
procedure ibwrtf   (boarddev:Integer; filename:PChar); stdcall;
procedure ibwrti   (boarddev:Integer; var writeval:word ; bytecount:Longint); stdcall;
procedure ibwrtia  (boarddev:Integer; var writeval:word ; bytecount:Longint); stdcall;

procedure AllSpoll (board:word; var adrlist:word; var result:word); stdcall;
procedure DevClear (board:word; adr:word); stdcall;
procedure DevClearList (board:word; var adrlist:word); stdcall;
procedure EnableLocal (board:word; var adrlist:word); stdcall;
procedure EnableRemote (board:word; var adrlist:word); stdcall;
procedure FindLstn (board:word; var pads:word; var results:word; limit:word); stdcall;
procedure FindRQS (board:word; var adrlist:word; var devstat:word); stdcall;
procedure PassControl (board:word; adr:word); stdcall;
procedure PPoll (board:word; var results:word); stdcall;
procedure PPollConfig (board:word; adr:word; dataline:word; linesense:word); stdcall;
procedure PPollUnconfig (board:word; var adrlist:word); stdcall;
procedure ReadStatusByte (board:word; adr:word; var result:word); stdcall;
procedure RcvRespMsg (board:word; buffer:PChar; bytecount:LongInt; eot:word); stdcall;
procedure Receive (board:word; adr:word; buffer:PChar; bytecount:LongInt; eot:word); stdcall;
procedure ReceiveSetup (board:word; adr:word); stdcall;
procedure ResetSys (board:word; var adrlist:word); stdcall;
procedure Send (board:word; adr:word; buffer:PChar; bytecount:LongInt; eot:word); stdcall;
procedure SendCmds (board:word; buffer:PChar; bytecount:LongInt); stdcall;
procedure SendIFC (board:word); stdcall;
procedure SendDataBytes (board:word; buffer:PChar; byteCount:LongInt; eot:word); stdcall;
procedure SendList (board:word; var adrlist:word; buffer:PChar; bytecount:LongInt; eot:word); stdcall;
procedure SendLLO (board:word); stdcall;
procedure SendSetup (board:word; var adrlist:word); stdcall;
procedure SetRWLS (board:word; var adrlist:word); stdcall;
procedure TestSRQ (board:word; var result:word); stdcall;
procedure TestSys (board:word; var adrlist:word; var result:word); stdcall;
procedure Trigger (board:word; adr:word); stdcall;
procedure TriggerList (board:word; var adrlist:word); stdcall;
procedure WaitSRQ (board:word; var result:word); stdcall;

{$ELSE}
{ 16 bit GPIB function prototypes }
procedure ibask    (board:Integer; item:Integer; var value:Integer);
procedure ibbna    (device:Integer; boardname:PChar);
procedure ibcac    (device:Integer; sync:Integer);
procedure ibclr    (device:Integer);
procedure ibcmd    (board:Integer; cmnd:PChar; bytecount:Longint);
procedure ibcmda   (board:Integer; cmnd:Pchar; bytecount:Longint);
procedure ibconfig (board:Integer; item:Integer; value:Integer);
function  ibdev    (boardindex:Integer; pad:Integer; sad:Integer;
                    timeout:Integer; eot:Integer; eos:Integer):Integer;
procedure ibdma    (device:Integer; dma:Integer);
procedure ibeos    (boarddev:Integer; eos:Integer);
procedure ibeot    (boarddev:Integer; eot:Integer);
procedure ibevent  (board:Integer; var event:Integer);
function  ibfind   (name:PChar):Integer;
procedure ibgts    (board:Integer; handshake:Integer);
procedure ibinit   (board:Integer);
procedure ibist    (board:Integer; Statusbit:Integer);
procedure iblines  (board:Integer; var clines:Integer);
procedure ibln     (board:Integer; pad:Integer; sad:Integer; var listenflag:Integer);
procedure ibloc    (boarddev:Integer);
procedure ibonl    (boarddev:Integer; online:Integer);
procedure ibpad    (boarddev:Integer; address:Integer);
procedure ibpct    (device:Integer);
procedure ibppc    (board:Integer; command:Integer);
procedure ibptrs   (var ibstaptr:word; var iberrPtr:word; var ibcntPtr:word; var ibcntlPtr:Longint);
procedure ibrd     (boarddev:Integer; readstr:Pchar; bytecount:Longint);
procedure ibrda    (boarddev:Integer; readstr:Pchar; bytecount:Longint);
procedure ibrdf    (boarddev:Integer; filename:PChar);
procedure ibrdi    (boarddev:Integer; var readval:word; bytecount:Longint);
procedure ibrdia   (boarddev:Integer; var readval:word; bytecount:Longint);
procedure ibrpp    (boarddev:Integer; var response:Byte);
procedure ibrsc    (board:Integer; control:Integer);
procedure ibrsp    (device:Integer; var serialpollbyte:Byte);
procedure ibrsv    (boarddev:Integer; statusbyte:Integer);
procedure ibsad    (boarddev:Integer; address:Integer);
procedure ibsic    (board:Integer);
procedure ibsre    (board:Integer; ren:Integer);
procedure ibstop   (boarddev:Integer);
procedure ibtmo    (boarddev:Integer; timeout:Integer);
procedure ibtrg    (device:Integer);
procedure ibwait   (boarddev:Integer; mask:word);
procedure ibwrt    (boarddev:Integer; writestr:Pchar; bytecount:Longint);
procedure ibwrta   (boarddev:Integer; writestr:Pchar; bytecount:Longint);
procedure ibwrtf   (boarddev:Integer; filename:PChar);
procedure ibwrti   (boarddev:Integer; var writeval:word ; bytecount:Longint);
procedure ibwrtia  (boarddev:Integer; var writeval:word ; bytecount:Longint);

procedure AllSpoll (board:word; var adrlist:word; var result:word);
procedure DevClear (board:word; adr:word);
procedure DevClearList (board:word; var adrlist:word);
procedure EnableLocal (board:word; var adrlist:word);
procedure EnableRemote (board:word; var adrlist:word);
procedure FindLstn (board:word; var pads:word; var results:word; limit:word);
procedure FindRQS (board:word; var adrlist:word; var devstat:word);
procedure PassControl (board:word; adr:word);
procedure PPoll (board:word; var results:word);
procedure PPollConfig (board:word; adr:word; dataline:word; linesense:word);
procedure PPollUnconfig (board:word; var adrlist:word);
procedure ReadStatusByte (board:word; adr:word; var result:word);
procedure RcvRespMsg (board:word; buffer:PChar; bytecount:LongInt; eot:word );
procedure Receive (board:word; adr:word; buffer:PChar; bytecount:LongInt; eot:word);
procedure ReceiveSetup (board:word; adr:word);
procedure ResetSys (board:word; var adrlist:word);
procedure Send (board:word; adr:word; buffer:PChar; bytecount:LongInt; eot:word);
procedure SendCmds (board:word; buffer:PChar; bytecount:LongInt);
procedure SendIFC (board:word);
procedure SendDataBytes (board:word; buffer:PChar; byteCount:LongInt; eot:word);
procedure SendList (board:word; var adrlist:word; buffer:PChar; bytecount:LongInt; eot:word);
procedure SendLLO (board:word);
procedure SendSetup (board:word; var adrlist:word);
procedure SetRWLS (board:word; var adrlist:word);
procedure TestSRQ (board:word; var result:word);
procedure TestSys (board:word; var adrlist:word; var result:word);
procedure Trigger (board:word; adr:word);
procedure TriggerList (board:word; var adrlist:word);
procedure WaitSRQ (board:word; var result:word);

{$ENDIF} {WIN32}


{$IFDEF WIN32}

implementation
   procedure ibask    (board:Integer; item:Integer; var value:Integer); external 'gpib-32.dll';
   procedure ibbna    (Device:Integer; BoardName:PChar); external 'gpib-32.dll';
   procedure ibcac    (Device:Integer; sync:Integer); external 'gpib-32.dll';
   procedure ibclr    (Device:Integer); external 'gpib-32.dll';
   procedure ibcmd    (Board:Integer; cmnd:PChar ; ByteCount:Longint); external 'gpib-32.dll';
   procedure ibcmda   (Board:Integer; cmnd:PChar; ByteCount:Longint); external 'gpib-32.dll';
   procedure ibconfig (Board:Integer; Item:Integer; Value:Integer); external 'gpib-32.dll';
   function  ibdev    (BoardIndex:Integer; pad:Integer; sad:Integer;
                       Timeout:Integer; EOT:Integer; EOS:Integer):Integer; external 'gpib-32.dll';
   procedure ibdma    (Device:Integer; DMA:Integer); external 'gpib-32.dll';
   procedure ibeos    (BoardDev:Integer; EOS:Integer); external 'gpib-32.dll';
   procedure ibeot    (BoardDev:Integer; EOT:Integer); external 'gpib-32.dll';
   procedure ibevent  (board:Integer; var event:Integer); external 'gpib-32.dll';
   function  ibfind   (Name:PChar):Integer; external 'gpib-32.dll';
   procedure ibgts    (Board:Integer; Handshake:Integer); external 'gpib-32.dll';
   procedure ibinit   (Board:Integer); external 'gpib-32.dll';
   procedure ibist    (Board:Integer; Statusbit:Integer); external 'gpib-32.dll';
   procedure iblines  (Board:Integer; var clines:Integer); external 'gpib-32.dll';
   procedure ibln     (board:Integer; pad:Integer; sad:Integer; var listenflag:Integer); external 'gpib-32.dll';
   procedure ibloc    (BoardDev:Integer); external 'gpib-32.dll';
   procedure ibonl    (BoardDev:Integer; OnLine:Integer); external 'gpib-32.dll';
   procedure ibpad    (BoardDev:Integer; Address:Integer); external 'gpib-32.dll';
   procedure ibpct    (Device:Integer); external 'gpib-32.dll';
   procedure ibppc    (Board:Integer; Command:Integer); external 'gpib-32.dll';
   procedure ibptrs   (var ibstaptr:word; var iberrPtr:word; var ibcntPtr:word; var ibcntlPtr:Longint); external 'gpib-32.dll'
   procedure ibrd     (BoardDev:Integer; ReadStr:Pchar; ByteCount:Longint); external 'gpib-32.dll';
   procedure ibrda    (BoardDev:Integer; ReadStr:Pchar; ByteCount:Longint); external 'gpib-32.dll';
   procedure ibrdf    (BoardDev:Integer; FileName:PChar); external 'gpib-32.dll';
   procedure ibrdi    (BoardDev:Integer; var ReadVal:word; ByteCount:Longint); external 'gpib-32.dll';
   procedure ibrdia   (BoardDev:Integer; var ReadVal:word; ByteCount:Longint); external 'gpib-32.dll';
   procedure ibrpp    (BoardDev:Integer; var Response:Byte); external 'gpib-32.dll';
   procedure ibrsc    (Board:Integer; Control:Integer); external 'gpib-32.dll';
   procedure ibrsp    (Device:Integer; var SerialPollByte:Byte); external 'gpib-32.dll';
   procedure ibrsv    (BoardDev:Integer; StatusByte:Integer); external 'gpib-32.dll';
   procedure ibsad    (BoardDev:Integer; Address:Integer); external 'gpib-32.dll';
   procedure ibsic    (Board:Integer); external 'gpib-32.dll';
   procedure ibsre    (Board:Integer; Ren:Integer); external 'gpib-32.dll';
   procedure ibstop   (BoardDev:Integer); external 'gpib-32.dll';
   procedure ibtmo    (BoardDev:Integer; Timeout:Integer); external 'gpib-32.dll';
   procedure ibtrg    (Device:Integer); external 'gpib-32.dll';
   procedure ibwait   (BoardDev:Integer; Mask:word); external 'gpib-32.dll';
   procedure ibwrt    (BoardDev:Integer; WriteStr:Pchar; ByteCount:Longint); external 'gpib-32.dll';
   procedure ibwrta   (BoardDev:Integer; WriteStr:Pchar; ByteCount:Longint); external 'gpib-32.dll';
   procedure ibwrtf   (BoardDev:Integer; FileName:PChar); external 'gpib-32.dll';
   procedure ibwrti   (BoardDev:Integer; var WriteVal:word; ByteCount:Longint); external 'gpib-32.dll';
   procedure ibwrtia  (BoardDev:Integer; var WriteVal:word; ByteCount:Longint); external 'gpib-32.dll';

   procedure AllSpoll (board:word; var adrlist:word; var result:word); external 'gpib-32.dll';
   procedure DevClear (board:word; adr:word); external 'gpib-32.dll';
   procedure DevClearList (board:word; var adrlist:word); external 'gpib-32.dll';
   procedure EnableLocal (board:word; var adrlist:word); external 'gpib-32.dll';
   procedure EnableRemote (board:word; var adrlist:word); external 'gpib-32.dll';
   procedure FindLstn (board:word; var pads:word; var results:word; limit:word); external 'gpib-32.dll';
   procedure FindRQS (board:word; var adrlist:word; var devstat:word); external 'gpib-32.dll';
   procedure PassControl (board:word; adr:word); external 'gpib-32.dll';
   procedure PPoll (board:word; var results:word); external 'gpib-32.dll';
   procedure PPollConfig (board:word; adr:word; dataline:word; linesense:word); external 'gpib-32.dll';
   procedure PPollUnconfig (board:word; var adrlist:word); external 'gpib-32.dll';
   procedure ReadStatusByte (board:word; adr:word; var result:word); external 'gpib-32.dll';
   procedure RcvRespMsg (board:word; buffer:PChar; bytecount:LongInt; eot:word); external 'gpib-32.dll';
   procedure Receive (board:word; adr:word; buffer:PChar; bytecount:LongInt; eot:word); external 'gpib-32.dll';
   procedure ReceiveSetup (board:word; adr:word); external 'gpib-32.dll';
   procedure ResetSys (board:word; var adrlist:word); external 'gpib-32.dll';
   procedure Send (board:word; adr:word; buffer:PChar; bytecount:LongInt; eot:word); external 'gpib-32.dll';
   procedure SendCmds (board:word; buffer:PChar; bytecount:LongInt); external 'gpib-32.dll';
   procedure SendIFC (board:word); external 'gpib-32.dll';
   procedure SendDataBytes (board:word; buffer:PChar; byteCount:LongInt; eot:word); external 'gpib-32.dll';
   procedure SendList (board:word; var adrlist:word; buffer:PChar; bytecount:LongInt; eot:word); external 'gpib-32.dll';
   procedure SendLLO (board:word); external 'gpib-32.dll';
   procedure SendSetup (board:word; var adrlist:word); external 'gpib-32.dll';
   procedure SetRWLS (board:word; var adrlist:word); external 'gpib-32.dll';
   procedure TestSRQ (board:word; var result:word); external 'gpib-32.dll';
   procedure TestSys (board:word; var adrlist:word; var result:word);external 'gpib-32.dll';
   procedure Trigger (board:word; adr:word);external 'gpib-32.dll';
   procedure TriggerList (board:word; var adrlist:word);external 'gpib-32.dll';
   procedure WaitSRQ (board:word; var result:word);external 'gpib-32.dll';
{$ELSE}
implementation
   procedure ibask    (board:Integer; item:Integer; var value:Integer); external 'gpib.dll';
   procedure ibbna    (Device:Integer; BoardName:PChar); external 'gpib.dll';
   procedure ibcac    (Device:Integer; sync:Integer); external 'gpib.dll';
   procedure ibclr    (Device:Integer); external 'gpib.dll';
   procedure ibcmd    (Board:Integer; cmnd:PChar ; ByteCount:Longint); external 'gpib.dll';
   procedure ibcmda   (Board:Integer; cmnd:PChar; ByteCount:Longint); external 'gpib.dll';
   procedure ibconfig (Board:Integer; Item:Integer; Value:Integer); external 'gpib.dll';
   function  ibdev    (BoardIndex:Integer; pad:Integer; sad:Integer;
                       Timeout:Integer; EOT:Integer; EOS:Integer):Integer; external 'gpib.dll';
   procedure ibdma    (Device:Integer; DMA:Integer); external 'gpib.dll';
   procedure ibeos    (BoardDev:Integer; EOS:Integer); external 'gpib.dll';
   procedure ibeot    (BoardDev:Integer; EOT:Integer); external 'gpib.dll';
   procedure ibevent  (board:Integer; var event:Integer); external 'gpib.dll';
   function  ibfind   (Name:PChar):Integer; external 'gpib.dll';
   procedure ibgts    (Board:Integer; Handshake:Integer); external 'gpib.dll';
   procedure ibinit   (Board:Integer); external 'gpib.dll';
   procedure ibist    (Board:Integer; Statusbit:Integer); external 'gpib.dll';
   procedure iblines  (Board:Integer; var clines:Integer); external 'gpib.dll';
   procedure ibln     (board:Integer; pad:Integer; sad:Integer; var listenflag:Integer); external 'gpib.dll';
   procedure ibloc    (BoardDev:Integer); external 'gpib.dll';
   procedure ibonl    (BoardDev:Integer; OnLine:Integer); external 'gpib.dll';
   procedure ibpad    (BoardDev:Integer; Address:Integer); external 'gpib.dll';
   procedure ibpct    (Device:Integer); external 'gpib.dll';
   procedure ibppc    (Board:Integer; Command:Integer); external 'gpib.dll';
   procedure ibptrs   (var ibstaptr:word; var iberrPtr:word; var ibcntPtr:word; var ibcntlPtr:Longint); external 'gpib.dll';
   procedure ibrd     (BoardDev:Integer; ReadStr:Pchar; ByteCount:Longint); external 'gpib.dll';
   procedure ibrda    (BoardDev:Integer; ReadStr:Pchar; ByteCount:Longint); external 'gpib.dll';
   procedure ibrdf    (BoardDev:Integer; FileName:PChar); external 'gpib.dll';
   procedure ibrdi    (BoardDev:Integer; var ReadVal:word; ByteCount:Longint); external 'gpib.dll';
   procedure ibrdia   (BoardDev:Integer; var ReadVal:word; ByteCount:Longint); external 'gpib.dll';
   procedure ibrpp    (BoardDev:Integer; var Response:Byte); external 'gpib.dll';
   procedure ibrsc    (Board:Integer; Control:Integer); external 'gpib.dll';
   procedure ibrsp    (Device:Integer; var SerialPollByte:Byte); external 'gpib.dll';
   procedure ibrsv    (BoardDev:Integer; StatusByte:Integer); external 'gpib.dll';
   procedure ibsad    (BoardDev:Integer; Address:Integer); external 'gpib.dll';
   procedure ibsic    (Board:Integer); external 'gpib.dll';
   procedure ibsre    (Board:Integer; Ren:Integer); external 'gpib.dll';
   procedure ibstop   (BoardDev:Integer); external 'gpib.dll';
   procedure ibtmo    (BoardDev:Integer; Timeout:Integer); external 'gpib.dll';
   procedure ibtrg    (Device:Integer); external 'gpib.dll';
   procedure ibwait   (BoardDev:Integer; Mask:word); external 'gpib.dll';
   procedure ibwrt    (BoardDev:Integer; WriteStr:Pchar; ByteCount:Longint); external 'gpib.dll';
   procedure ibwrta   (BoardDev:Integer; WriteStr:Pchar; ByteCount:Longint); external 'gpib.dll';
   procedure ibwrtf   (BoardDev:Integer; FileName:PChar); external 'gpib.dll';
   procedure ibwrti   (BoardDev:Integer; var WriteVal:word; ByteCount:Longint); external 'gpib.dll';
   procedure ibwrtia  (BoardDev:Integer; var WriteVal:word; ByteCount:Longint); external 'gpib.dll';

   procedure AllSpoll (board:word; var adrlist:word; var result:word); external 'gpib.dll';
   procedure DevClear (board:word; adr:word); external 'gpib.dll';
   procedure DevClearList (board:word; var adrlist:word); external 'gpib.dll';
   procedure EnableLocal (board:word; var adrlist:word); external 'gpib.dll';
   procedure EnableRemote (board:word; var adrlist:word); external 'gpib.dll';
   procedure FindLstn (board:word; var pads:word; var results:word; limit:word); external 'gpib.dll';
   procedure FindRQS (board:word; var adrlist:word; var devstat:word); external 'gpib.dll';
   procedure PassControl (board:word; adr:word); external 'gpib.dll';
   procedure PPoll (board:word; var results:word); external 'gpib.dll';
   procedure PPollConfig (board:word; adr:word; dataline:word; linesense:word); external 'gpib.dll';
   procedure PPollUnconfig (board:word; var adrlist:word); external 'gpib.dll';
   procedure ReadStatusByte (board:word; adr:word; var result:word); external 'gpib.dll';
   procedure RcvRespMsg (board:word; buffer:PChar; bytecount:LongInt; eot:word ); external 'gpib.dll';
   procedure Receive (board:word; adr:word; buffer:PChar; bytecount:LongInt; eot:word); external 'gpib.dll';
   procedure ReceiveSetup (board:word; adr:word); external 'gpib.dll';
   procedure ResetSys (board:word; var adrlist:word); external 'gpib.dll';
   procedure Send (board:word; adr:word; buffer:PChar; bytecount:LongInt; eot:word); external 'gpib.dll';
   procedure SendCmds (board:word; buffer:PChar; bytecount:LongInt); external 'gpib.dll';
   procedure SendIFC (board:word); external 'gpib.dll';
   procedure SendDataBytes (board:word; buffer:PChar; byteCount:LongInt; eot:word); external 'gpib.dll';
   procedure SendList (board:word; var adrlist:word; buffer:PChar; bytecount:LongInt; eot:word); external 'gpib.dll';
   procedure SendLLO (board:word); external 'gpib.dll';
   procedure SendSetup (board:word; var adrlist:word); external 'gpib.dll';
   procedure SetRWLS (board:word; var adrlist:word); external 'gpib.dll';
   procedure TestSRQ (board:word; var result:word); external 'gpib.dll';
   procedure TestSys (board:word; var adrlist:word; var result:word); external 'gpib.dll';
   procedure Trigger (board:word; adr:word); external 'gpib.dll';
   procedure TriggerList (board:word; var adrlist:word); external 'gpib.dll';
   procedure WaitSRQ (board:word; var result:word); external 'gpib.dll';

{$ENDIF} {WIN32}
end
.


